@@ -1,12 +1,12 @@
 
-
+/* Generated data (by glib-mkenums) */
 
 #include "xfce-notify-enum-types.h"
 #include "xfce-notify-window.h"
 
-/* enumerations from "xfce-notify-window.h" */
+/* enumerations from "xfce4-notifyd/xfce-notify-window.h" */
 GType
-xfce_notify_close_reason_get_type()
+xfce_notify_close_reason_get_type(void)
 {
 	static GType type = 0;
 	if(type == 0) {
@@ -22,5 +22,5 @@ xfce_notify_close_reason_get_type()
 	return type;
 }
 
-
+/* Generated data ends here */
 
