@@ -1,7 +1,8 @@
 /*
  *  xfce4-notifyd
  *
- *  Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2008-2009 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2009 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,11 +28,7 @@
 
 #include <math.h>
 
-#include <libxfcegui4/libxfcegui4.h>
-
-#ifdef HAVE_LIBSEXY
-#include <libsexy/sexy.h>
-#endif
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "xfce-notify-window.h"
 #include "xfce-notify-enum-types.h"
@@ -39,15 +36,19 @@
 #define DEFAULT_EXPIRE_TIMEOUT    10000
 #define DEFAULT_NORMAL_OPACITY        0.85
 #define FADE_TIME                   800
-#define FADE_CHANGE_TIMEOUT          50 
+#define FADE_CHANGE_TIMEOUT          50
 #define DEFAULT_RADIUS               10.0
 #define DEFAULT_BORDER_WIDTH          2.0
-#define BORDER                        8
+#define BORDER                        6
 
 struct _XfceNotifyWindow
 {
     GtkWindow parent;
 
+    GdkRectangle geometry;
+    gint last_monitor;
+    gint last_screen;
+
     guint expire_timeout;
 
     gboolean mouse_hover;
@@ -55,25 +56,27 @@ struct _XfceNotifyWindow
     cairo_path_t *close_btn_path;
     GdkRegion *close_btn_region;
 
-    gboolean fade_transparent;
     gdouble normal_opacity;
 
+    guint32 fade_transparent:1,
+            icon_only:1,
+            has_summary_text:1,
+            has_body_text:1,
+            has_actions;
+
+    GtkWidget *icon_box;
     GtkWidget *icon;
+    GtkWidget *content_box;
+    GtkWidget *gauge;
     GtkWidget *summary;
     GtkWidget *body;
     GtkWidget *button_box;
-    
+
     guint64 expire_start_timestamp;
     guint expire_id;
     guint fade_id;
     guint op_change_steps;
     gdouble op_change_delta;
-
-    gboolean maybe_begin_drag;
-    gboolean dragging;
-    gint press_start_x;
-    gint press_start_y;
-    guint32 press_timestamp;
 };
 
 typedef struct
@@ -94,9 +97,6 @@ enum
     N_SIGS,
 };
 
-static void xfce_notify_window_class_init(XfceNotifyWindowClass *klass);
-static void xfce_notify_window_init(XfceNotifyWindow *window);
-
 static void xfce_notify_window_finalize(GObject *object);
 
 static void xfce_notify_window_realize(GtkWidget *widget);
@@ -105,14 +105,12 @@ static gboolean xfce_notify_window_expose(GtkWidget *widget,
                                           GdkEventExpose *evt);
 static gboolean xfce_notify_window_enter_leave(GtkWidget *widget,
                                                GdkEventCrossing *evt);
-static gboolean xfce_notify_window_button_press(GtkWidget *widget,
-                                                GdkEventButton *evt);
 static gboolean xfce_notify_window_button_release(GtkWidget *widget,
                                                   GdkEventButton *evt);
-static gboolean xfce_notify_window_motion_notify(GtkWidget *widget,
-                                                 GdkEventMotion *evt);
 static gboolean xfce_notify_window_configure_event(GtkWidget *widget,
                                                    GdkEventConfigure *evt);
+static void xfce_notify_window_style_set(GtkWidget *widget,
+                                         GtkStyle *previous_style);
 
 static gboolean xfce_notify_window_expire_timeout(gpointer data);
 static gboolean xfce_notify_window_fade_timeout(gpointer data);
@@ -120,12 +118,6 @@ static gboolean xfce_notify_window_fade_timeout(gpointer data);
 static void xfce_notify_window_button_clicked(GtkWidget *widget,
                                               gpointer user_data);
 
-#ifdef HAVE_LIBSEXY
-static void xfce_notify_window_url_clicked(SexyUrlLabel *label,
-                                           const gchar *url,
-                                           gpointer user_data);
-#endif
-
 static guint signals[N_SIGS] = { 0, };
 
 
@@ -145,10 +137,9 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
     widget_class->expose_event = xfce_notify_window_expose;
     widget_class->enter_notify_event = xfce_notify_window_enter_leave;
     widget_class->leave_notify_event = xfce_notify_window_enter_leave;
-    widget_class->button_press_event = xfce_notify_window_button_press;
     widget_class->button_release_event = xfce_notify_window_button_release;
-    widget_class->motion_notify_event = xfce_notify_window_motion_notify;
     widget_class->configure_event = xfce_notify_window_configure_event;
+    widget_class->style_set = xfce_notify_window_style_set;
 
     signals[SIG_CLOSED] = g_signal_new("closed",
                                        XFCE_TYPE_NOTIFY_WINDOW,
@@ -191,18 +182,27 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
                                                                 0.0, 8.0,
                                                                 DEFAULT_BORDER_WIDTH,
                                                                 G_PARAM_READABLE));
+    gtk_widget_class_install_style_property(widget_class,
+                                            g_param_spec_boolean("summary-bold",
+                                                                 "summary bold",
+                                                                 "whether or not to display the notification summary field in bold text",
+                                                                 FALSE,
+                                                                 G_PARAM_READABLE));
 }
 
 static void
 xfce_notify_window_init(XfceNotifyWindow *window)
 {
     GdkScreen *screen;
-    GtkWidget *topvbox, *vbox, *hbox, *spacer;
-    
+    GtkWidget *tophbox, *align, *vbox;
+    gdouble border_radius = DEFAULT_RADIUS;
+
     GTK_WINDOW(window)->type = GTK_WINDOW_TOPLEVEL;
     window->expire_timeout = DEFAULT_EXPIRE_TIMEOUT;
     window->normal_opacity = DEFAULT_NORMAL_OPACITY;
     window->fade_transparent = TRUE;
+    /* The summary widget needs to be initialized before style_set is called. gtk_widget_ensure_style calls style_set */
+    window->summary = gtk_label_new(NULL);
 
     gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
     gtk_window_stick(GTK_WINDOW(window));
@@ -213,12 +213,12 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
     gtk_window_set_type_hint(GTK_WINDOW(window),
                              GDK_WINDOW_TYPE_HINT_NOTIFICATION);
-    gtk_container_set_border_width(GTK_CONTAINER(window), BORDER);
+    gtk_container_set_border_width(GTK_CONTAINER(window), 0);
     gtk_widget_set_app_paintable(GTK_WIDGET(window), TRUE);
     gtk_widget_add_events(GTK_WIDGET(window),
                           GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                           | GDK_POINTER_MOTION_MASK);
-    
+
     screen = gtk_widget_get_screen(GTK_WIDGET(window));
     if(gdk_screen_is_composited(screen)) {
         GdkColormap *cmap = gdk_screen_get_rgba_colormap(screen);
@@ -226,55 +226,44 @@ xfce_notify_window_init(XfceNotifyWindow *window)
             gtk_widget_set_colormap(GTK_WIDGET(window), cmap);
     }
 
-    topvbox = gtk_vbox_new(FALSE, BORDER);
-    gtk_widget_show(topvbox);
-    gtk_container_add(GTK_CONTAINER(window), topvbox);
+    gtk_widget_ensure_style(GTK_WIDGET(window));
+    gtk_widget_style_get(GTK_WIDGET(window),
+                         "border-radius", &border_radius,
+                         NULL);
 
-    spacer = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_widget_set_size_request(spacer, -1, 20 - BORDER * 2);
-    gtk_widget_show(spacer);
-    gtk_box_pack_start(GTK_BOX(topvbox), spacer, FALSE, FALSE, 0);
+    tophbox = gtk_hbox_new(FALSE, BORDER);
+    gtk_container_set_border_width(GTK_CONTAINER(tophbox), border_radius + 4);
+    gtk_widget_show(tophbox);
+    gtk_container_add(GTK_CONTAINER(window), tophbox);
 
-    hbox = gtk_hbox_new(FALSE, BORDER);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(topvbox), hbox, TRUE, TRUE, 0);
-
-    spacer = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_widget_set_size_request(spacer, 20 - BORDER * 2, -1);
-    gtk_widget_show(spacer);
-    gtk_box_pack_start(GTK_BOX(hbox), spacer, FALSE, FALSE, 0);
+    window->icon_box = align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
+    gtk_container_set_border_width(GTK_CONTAINER(align), 0);
+    gtk_box_pack_start(GTK_BOX(tophbox), align, FALSE, TRUE, 0);
 
     window->icon = gtk_image_new();
-    gtk_box_pack_start(GTK_BOX(hbox), window->icon, FALSE, FALSE, 0);
+    gtk_widget_show(window->icon);
+    gtk_container_add(GTK_CONTAINER(align), window->icon);
 
-    vbox = gtk_vbox_new(FALSE, BORDER);
+    window->content_box = vbox = gtk_vbox_new(FALSE, BORDER);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
     gtk_widget_show(vbox);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(tophbox), vbox, TRUE, TRUE, 0);
 
-    window->summary = gtk_label_new(NULL);
     gtk_label_set_line_wrap(GTK_LABEL(window->summary), TRUE);
-    gtk_misc_set_alignment(GTK_MISC(window->summary), 0.0, 0.5);
+    gtk_misc_set_alignment(GTK_MISC(window->summary), 0.0, 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), window->summary, FALSE, FALSE, 0);
 
-#ifdef HAVE_LIBSEXY
-    window->body = sexy_url_label_new();
-#else
     window->body = gtk_label_new(NULL);
-#endif
     gtk_label_set_line_wrap(GTK_LABEL(window->body), TRUE);
-    gtk_misc_set_alignment(GTK_MISC(window->body), 0.0, 0.5);
+    gtk_misc_set_alignment(GTK_MISC(window->body), 0.0, 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), window->body, TRUE, TRUE, 0);
-#ifdef HAVE_LIBSEXY
-    g_signal_connect(G_OBJECT(window->body), "url-activated",
-                     G_CALLBACK(xfce_notify_window_url_clicked), window);
-#endif
 
     window->button_box = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(window->button_box),
                               GTK_BUTTONBOX_END);
     gtk_box_set_spacing(GTK_BOX(window->button_box), BORDER / 2);
     gtk_box_set_homogeneous(GTK_BOX(window->button_box), FALSE);
-    gtk_box_pack_start(GTK_BOX(topvbox), window->button_box, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), window->button_box, FALSE, FALSE, 0);
 }
 
 static void
@@ -445,7 +434,7 @@ xfce_notify_window_ensure_bg_path(XfceNotifyWindow *window,
     }
 
     window->bg_path = cairo_copy_path(cr);
-    
+
     flat_path = cairo_copy_path_flat(cr);
     fill_rule = (cairo_get_fill_rule(cr) == CAIRO_FILL_RULE_WINDING
                  ? GDK_WINDING_RULE : GDK_EVEN_ODD_RULE);
@@ -533,7 +522,7 @@ xfce_notify_window_expose(GtkWidget *widget,
             cairo_path_t *flat_path;
             GdkFillRule fill_rule;
 
-            cairo_arc(cr, 12., 12., 7.5, 0., 2*M_PI);
+            cairo_arc(cr, widget->allocation.width - 12., 12., 7.5, 0., 2*M_PI);
             window->close_btn_path = cairo_copy_path(cr);
 
             flat_path = cairo_copy_path_flat(cr);
@@ -547,11 +536,11 @@ xfce_notify_window_expose(GtkWidget *widget,
         cairo_set_line_width(cr, 1.5);
         cairo_stroke(cr);
 
-        cairo_move_to(cr, 8., 8.);
-        cairo_line_to(cr, 16., 16.);
+        cairo_move_to(cr, widget->allocation.width - 8., 8.);
+        cairo_line_to(cr, widget->allocation.width - 16., 16.);
         cairo_stroke(cr);
-        cairo_move_to(cr, 16., 8.);
-        cairo_line_to(cr, 8., 16.);
+        cairo_move_to(cr, widget->allocation.width - 16., 8.);
+        cairo_line_to(cr, widget->allocation.width - 8., 16.);
         cairo_stroke(cr);
     }
 
@@ -572,11 +561,6 @@ xfce_notify_window_enter_leave(GtkWidget *widget,
     XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
 
     if(evt->type == GDK_ENTER_NOTIFY) {
-        /* after a drag, we don't get a button release, but we do get an
-         * enter notify */
-        window->maybe_begin_drag = FALSE;
-        window->dragging = FALSE;
-
         if(window->expire_timeout) {
             if(window->expire_id) {
                 g_source_remove(window->expire_id);
@@ -591,8 +575,7 @@ xfce_notify_window_enter_leave(GtkWidget *widget,
         window->mouse_hover = TRUE;
         gtk_widget_queue_draw(widget);
     } else if(evt->type == GDK_LEAVE_NOTIFY
-              && evt->detail != GDK_NOTIFY_INFERIOR
-              && !window->dragging)
+              && evt->detail != GDK_NOTIFY_INFERIOR)
     {
         xfce_notify_window_start_expiration(window);
         window->mouse_hover = FALSE;
@@ -602,26 +585,6 @@ xfce_notify_window_enter_leave(GtkWidget *widget,
     return FALSE;
 }
 
-static gboolean
-xfce_notify_window_button_press(GtkWidget *widget,
-                                GdkEventButton *evt)
-{
-    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
-
-    if(GDK_BUTTON_PRESS == evt->type && 1 == evt->button) {
-        window->maybe_begin_drag = TRUE;
-        window->dragging = FALSE;
-        window->press_start_x = evt->x_root;
-        window->press_start_y = evt->y_root;
-        window->press_timestamp = evt->time;
-    } else {
-        window->maybe_begin_drag = FALSE;
-        window->dragging = FALSE;
-    }
-
-    return FALSE;
-}
-
 static gboolean
 xfce_notify_window_button_release(GtkWidget *widget,
                                   GdkEventButton *evt)
@@ -641,28 +604,6 @@ xfce_notify_window_button_release(GtkWidget *widget,
     return FALSE;
 }
 
-static gboolean
-xfce_notify_window_motion_notify(GtkWidget *widget,
-                                 GdkEventMotion *evt)
-{
-    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
-
-    if(window->maybe_begin_drag) {
-        if(gtk_drag_check_threshold(widget, window->press_start_x,
-                                    window->press_start_y,
-                                    evt->x_root, evt->y_root))
-        {
-            window->dragging = TRUE;
-            gtk_window_begin_move_drag(GTK_WINDOW(widget), 1,
-                                       window->press_start_x,
-                                       window->press_start_y,
-                                       window->press_timestamp);
-        }
-    }
-
-    return FALSE;
-}
-
 static gboolean
 xfce_notify_window_configure_event(GtkWidget *widget,
                                    GdkEventConfigure *evt)
@@ -693,6 +634,27 @@ xfce_notify_window_configure_event(GtkWidget *widget,
     return ret;
 }
 
+static void
+xfce_notify_window_style_set(GtkWidget *widget,
+                             GtkStyle *previous_style)
+{
+    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
+    gboolean summary_bold = FALSE;
+    GtkStyle *style;
+    PangoFontDescription *pfd;
+
+    gtk_widget_style_get(widget,
+                         "summary-bold", &summary_bold,
+                         NULL);
+    if(summary_bold) {
+        style = gtk_widget_get_style(window->summary);
+        pfd = pango_font_description_copy(style->font_desc);
+        pango_font_description_set_weight(pfd, PANGO_WEIGHT_BOLD);
+        gtk_widget_modify_font(window->summary, pfd);
+        pango_font_description_free(pfd);
+    }
+}
+
 
 
 static gboolean
@@ -753,30 +715,6 @@ xfce_notify_window_button_clicked(GtkWidget *widget,
                   XFCE_NOTIFY_CLOSE_REASON_DISMISSED);
 }
 
-#ifdef HAVE_LIBSEXY
-static void
-xfce_notify_window_url_clicked(SexyUrlLabel *label,
-                               const gchar *url,
-                               gpointer user_data)
-{
-    gchar *opener, *url_quoted, *cmd = NULL;
-
-    if(!(opener = g_find_program_in_path("xdg-open")))
-        if(!(opener = g_find_program_in_path("exo-open")))
-            if(!(opener = g_find_program_in_path("gnome-open")))
-                opener = g_find_program_in_path("firefox");
-
-    if(opener) {
-        url_quoted = g_shell_quote(url);
-        cmd = g_strdup_printf("%s %s", opener, url_quoted);
-        xfce_exec(cmd, FALSE, FALSE, NULL);
-        g_free(url_quoted);
-        g_free(cmd);
-        g_free(opener);
-    }
-}
-#endif
-
 #define ELEM_B    GUINT_TO_POINTER(1)
 #define ELEM_I    GUINT_TO_POINTER(2)
 #define ELEM_U    GUINT_TO_POINTER(3)
@@ -839,8 +777,8 @@ xfce_notify_window_validate_escape_markup(const gchar *str)
                     g_warning("Bad markup in <a>: %s", str);
                     goto out_err;
                 }
-#ifdef HAVE_LIBSEXY
-                /* only support links with SexyUrlLabel*/
+#if GTK_CHECK_VERSION(2,16,0)
+                /* only support links with gtk 2.16 */
                 g_string_append_len(gstr, p, aend - p + 1);
 #endif
                 p = aend + 1;
@@ -920,7 +858,7 @@ xfce_notify_window_validate_escape_markup(const gchar *str)
                                   elem_to_string(tmp));
                         goto out_err;
                     }
-#ifdef HAVE_LIBSEXY
+#if GTK_CHECK_VERSION(2,16,0)
                     g_string_append(gstr, "</a>");
 #endif
                     p += 4;
@@ -963,7 +901,7 @@ out_err:
 
 
 GtkWidget *
-xfce_notify_window_new()
+xfce_notify_window_new(void)
 {
     return xfce_notify_window_new_with_actions(NULL, NULL, NULL, -1, NULL);
 }
@@ -990,13 +928,13 @@ xfce_notify_window_new_with_actions(const gchar *summary,
     XfceNotifyWindow *window;
 
     window = g_object_new(XFCE_TYPE_NOTIFY_WINDOW, NULL);
-    
+
     xfce_notify_window_set_summary(window, summary);
     xfce_notify_window_set_body(window, body);
     xfce_notify_window_set_icon_name(window, icon_name);
     xfce_notify_window_set_expire_timeout(window, expire_timeout);
     xfce_notify_window_set_actions(window, actions);
-    
+
     return GTK_WIDGET(window);
 }
 
@@ -1005,12 +943,15 @@ xfce_notify_window_set_summary(XfceNotifyWindow *window,
                                const gchar *summary)
 {
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
-    
+
     gtk_label_set_text(GTK_LABEL(window->summary), summary);
-    if(summary && *summary)
+    if(summary && *summary) {
         gtk_widget_show(window->summary);
-    else
+        window->has_summary_text = TRUE;
+    } else {
         gtk_widget_hide(window->summary);
+        window->has_summary_text = FALSE;
+    }
 
     if(window->bg_path) {
         cairo_path_destroy(window->bg_path);
@@ -1024,22 +965,21 @@ xfce_notify_window_set_body(XfceNotifyWindow *window,
                             const gchar *body)
 {
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
-    
+
     if(body && *body) {
         gchar *markup = xfce_notify_window_validate_escape_markup(body);
         if(!markup)
             return;
-#ifdef HAVE_LIBSEXY
-        sexy_url_label_set_markup(SEXY_URL_LABEL(window->body), markup);
-#else
+
         gtk_label_set_markup(GTK_LABEL(window->body), markup);
-        gtk_label_set_use_markup(GTK_LABEL(window->body), TRUE);
-#endif
+
         gtk_widget_show(window->body);
         g_free(markup);
+        window->has_body_text = TRUE;
     } else {
         gtk_label_set_markup(GTK_LABEL(window->body), "");
         gtk_widget_hide(window->body);
+        window->has_body_text = FALSE;
     }
 
     if(window->bg_path) {
@@ -1049,31 +989,75 @@ xfce_notify_window_set_body(XfceNotifyWindow *window,
     }
 }
 
+void
+xfce_notify_window_set_geometry(XfceNotifyWindow *window,
+                                GdkRectangle rectangle)
+{
+    window->geometry = rectangle;
+}
+
+GdkRectangle *
+xfce_notify_window_get_geometry (XfceNotifyWindow *window)
+{
+   return &window->geometry;
+}
+
+void
+xfce_notify_window_set_last_monitor(XfceNotifyWindow *window,
+                                    gint monitor)
+{
+    window->last_monitor = monitor;
+}
+
+gint
+xfce_notify_window_get_last_monitor(XfceNotifyWindow *window)
+{
+   return window->last_monitor;
+}
+
+void
+xfce_notify_window_set_last_screen(XfceNotifyWindow *window,
+                                   gint screen)
+{
+    window->last_screen = screen;
+}
+
+gint
+xfce_notify_window_get_last_screen(XfceNotifyWindow *window)
+{
+    return window->last_screen;
+}
+
 void
 xfce_notify_window_set_icon_name(XfceNotifyWindow *window,
                                  const gchar *icon_name)
 {
     gboolean icon_set = FALSE;
-    
+
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
     if(icon_name && *icon_name) {
         gint w, h;
         GdkPixbuf *pix;
-        
+
         gtk_icon_size_lookup(GTK_ICON_SIZE_DIALOG, &w, &h);
-        pix = xfce_themed_icon_load(icon_name, w);
+        pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
+                                       icon_name,
+                                       w,
+                                       GTK_ICON_LOOKUP_FORCE_SIZE,
+                                       NULL);
+
         if(pix) {
             gtk_image_set_from_pixbuf(GTK_IMAGE(window->icon), pix);
-            gtk_widget_show(window->icon);
+            gtk_widget_show(window->icon_box);
             g_object_unref(G_OBJECT(pix));
             icon_set = TRUE;
         }
     }
-    
+
     if(!icon_set) {
         gtk_image_set_from_pixbuf(GTK_IMAGE(window->icon), NULL);
-        gtk_widget_hide(window->icon);
+        gtk_widget_hide(window->icon_box);
     }
 
     if(window->bg_path) {
@@ -1120,9 +1104,9 @@ xfce_notify_window_set_icon_pixbuf(XfceNotifyWindow *window,
     gtk_image_set_from_pixbuf(GTK_IMAGE(window->icon), pixbuf);
 
     if(pixbuf)
-        gtk_widget_show(window->icon);
+        gtk_widget_show(window->icon_box);
     else
-        gtk_widget_hide(window->icon);
+        gtk_widget_hide(window->icon_box);
 
     if(window->bg_path) {
         cairo_path_destroy(window->bg_path);
@@ -1168,10 +1152,13 @@ xfce_notify_window_set_actions(XfceNotifyWindow *window,
         gtk_widget_destroy(GTK_WIDGET(l->data));
     g_list_free(children);
 
-    if(!actions)
+    if(!actions) {
         gtk_widget_hide(window->button_box);
-    else
+        window->has_actions = FALSE;
+    } else {
         gtk_widget_show(window->button_box);
+        window->has_actions = TRUE;
+    }
 
     for(i = 0; actions && actions[i]; i += 2) {
         const gchar *cur_action_id = actions[i];
@@ -1220,7 +1207,7 @@ xfce_notify_window_set_fade_transparent(XfceNotifyWindow *window,
     if(fade_transparent == window->fade_transparent)
         return;
 
-    window->fade_transparent = fade_transparent;
+    window->fade_transparent = !!fade_transparent;
 
     /* if we're already realized, we don't actually do anything here */
 }
@@ -1258,6 +1245,108 @@ xfce_notify_window_get_opacity(XfceNotifyWindow *window)
     return window->normal_opacity;
 }
 
+void
+xfce_notify_window_set_icon_only(XfceNotifyWindow *window,
+                                 gboolean icon_only)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    if(icon_only == window->icon_only)
+        return;
+
+    window->icon_only = !!icon_only;
+
+    if(icon_only) {
+        GtkRequisition req;
+
+        if(!GTK_WIDGET_VISIBLE(window->icon_box)) {
+            g_warning("Attempt to set icon-only mode with no icon");
+            return;
+        }
+
+        gtk_widget_hide(window->content_box);
+
+        /* set a wider size on the icon box so it takes up more space */
+        gtk_widget_realize(window->icon);
+        gtk_widget_size_request(window->icon, &req);
+        gtk_widget_set_size_request(window->icon_box, req.width * 4, -1);
+        /* and center it */
+        gtk_alignment_set(GTK_ALIGNMENT(window->icon_box), 0.5, 0.0, 0.0, 0.0);
+    } else {
+        gtk_alignment_set(GTK_ALIGNMENT(window->icon_box), 0.0, 0.0, 0.0, 0.0);
+        gtk_widget_set_size_request(window->icon_box, -1, -1);
+        gtk_widget_show(window->content_box);
+    }
+}
+
+void
+xfce_notify_window_set_gauge_value(XfceNotifyWindow *window,
+                                   gint value)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    /* maybe want to do some kind of effect if the value is out of bounds */
+    if(value > 100)
+        value = 100;
+    else if(value < 0)
+        value = 0;
+
+    gtk_widget_hide(window->summary);
+    gtk_widget_hide(window->body);
+    gtk_widget_hide(window->button_box);
+
+    if(!window->gauge) {
+        GtkWidget *align;
+        gint width;
+
+        if(GTK_WIDGET_VISIBLE(window->icon)) {
+            /* size the pbar in relation to the icon */
+            GtkRequisition req;
+
+            gtk_widget_realize(window->icon);
+            gtk_widget_size_request(window->icon, &req);
+            width = req.width * 4;
+        } else {
+            /* FIXME: do something less arbitrary */
+            width = 120;
+        }
+
+        align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
+        gtk_widget_show(align);
+        gtk_box_pack_start(GTK_BOX(window->content_box), align, TRUE, TRUE, 0);
+
+        window->gauge = gtk_progress_bar_new();
+        gtk_widget_set_size_request(window->gauge, width, -1);
+        gtk_widget_show(window->gauge);
+        gtk_container_add(GTK_CONTAINER(align), window->gauge);
+    }
+
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->gauge),
+                                  value / 100.0);
+}
+
+void
+xfce_notify_window_unset_gauge_value(XfceNotifyWindow *window)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    if(window->gauge) {
+        GtkWidget *align = gtk_widget_get_parent(window->gauge);
+
+        g_assert(align);
+
+        gtk_widget_destroy(align);
+        window->gauge = NULL;
+
+        if(window->has_summary_text)
+            gtk_widget_show(window->summary);
+        if(window->has_body_text)
+            gtk_widget_show(window->body);
+        if(window->has_actions)
+            gtk_widget_show(window->button_box);
+    }
+}
+
 void
 xfce_notify_window_closed(XfceNotifyWindow *window,
                           XfceNotifyCloseReason reason)
